@@ -24,6 +24,7 @@
 
 #include <config.h>
 #include "itdb.h"
+#include "itdb_private.h"
 #include "db-artwork-parser.h"
 
 #if HAVE_GDKPIXBUF
@@ -31,6 +32,7 @@
 #include "db-artwork-debug.h"
 #include "db-itunes-parser.h"
 #include "db-image-parser.h"
+#include "itdb_endianness.h"
 
 #include <errno.h>
 #include <fcntl.h>
@@ -57,6 +59,8 @@ struct iPodMmapBuffer {
 struct _iPodBuffer {
 	struct iPodMmapBuffer *mmap;
 	off_t offset;
+	guint byte_order;
+	DbType db_type;
 };
 
 typedef struct _iPodBuffer iPodBuffer;
@@ -94,14 +98,14 @@ ipod_buffer_grow_file (struct iPodMmapBuffer *mmap_buf, off_t new_size)
 	result = lseek (mmap_buf->fd, new_size, SEEK_SET);
 	if (result == (off_t)-1) {
 		g_print ("Failed to grow file to %lu: %s\n", 
-			 new_size, strerror (errno));
+                        (unsigned long)new_size, strerror (errno));
 		return -1;
 	}
 	result = 0;
 	result = write (mmap_buf->fd, &result, 1);
 	if (result != 1) {
 		g_print ("Failed to write a byte at %lu: %s\n", 
-			 new_size, strerror (errno));
+                        (unsigned long)new_size, strerror (errno));
 		return -1;
 	}
 
@@ -172,6 +176,8 @@ ipod_buffer_get_sub_buffer (iPodBuffer *buffer, off_t offset)
 	}
 	sub_buffer->mmap = buffer->mmap;
 	sub_buffer->offset = buffer->offset + offset;
+	sub_buffer->byte_order = buffer->byte_order;
+	sub_buffer->db_type = buffer->db_type;
 
 	buffer->mmap->ref_count++;
 
@@ -179,7 +185,7 @@ ipod_buffer_get_sub_buffer (iPodBuffer *buffer, off_t offset)
 }
 
 static iPodBuffer *
-ipod_buffer_new (const char *filename)
+ipod_buffer_new (const char *filename, guint byte_order, DbType db_type)
 {
 	int fd;
 	struct iPodMmapBuffer *mmap_buf;
@@ -224,6 +230,8 @@ ipod_buffer_new (const char *filename)
 		return NULL;
 	}
 	buffer->mmap = mmap_buf;
+	buffer->byte_order = byte_order;
+	buffer->db_type = db_type;
 
 	return buffer;
 }
@@ -235,13 +243,12 @@ enum MhsdType {
 };
 
 enum iPodThumbnailType {
-	IPOD_THUMBNAIL_FULL_SIZE,
-	IPOD_THUMBNAIL_NOW_PLAYING
+	IPOD_THUMBNAIL_FULL_SIZE = 0,
+	IPOD_THUMBNAIL_NOW_PLAYING = 1,
+	IPOD_PHOTO_LITTLE = 3,
+	IPOD_PHOTO_FULL_SIZE = 4
 };
 
-#define IPOD_THUMBNAIL_FULL_SIZE_SIZE (140*140*2)
-#define IPOD_THUMBNAIL_NOW_PLAYING_SIZE (56*56*2)
-
 
 #define RETURN_SIZE_FOR(id, size) if (strncmp (id, header_id, 4) == 0) return (size)
 
@@ -261,17 +268,19 @@ get_padded_header_size (gchar header_id[4])
 	RETURN_SIZE_FOR ("mhla", 0x5c);
 	RETURN_SIZE_FOR ("mhlf", 0x5c);
 	RETURN_SIZE_FOR ("mhif", 0x7c);
+	RETURN_SIZE_FOR ("mhba", 0x94);
 
 	return 0;
 }
 
 static void *
-init_header (iPodBuffer *buffer, gchar header_id[4], guint header_len)
+init_header (iPodBuffer *buffer, gchar _header_id[4], guint header_len)
 {
 	MHeader *mh;
 	int padded_size;
+	gchar *header_id;
 
-	padded_size = get_padded_header_size (header_id);
+	padded_size = get_padded_header_size (_header_id);
 	if (padded_size != 0) {
 		header_len = padded_size;
 	}
@@ -281,58 +290,135 @@ init_header (iPodBuffer *buffer, gchar header_id[4], guint header_len)
 	}
 	mh = (MHeader*)ipod_buffer_get_pointer (buffer);
 	memset (mh, 0, header_len);
+
+	header_id = g_strndup (_header_id, 4);
+	if (buffer->byte_order == G_BIG_ENDIAN) {
+		g_strreverse (header_id);
+	}
 	strncpy ((char *)mh->header_id, header_id, 4);
-	mh->header_len = GINT_TO_LE (header_len);
-	
+	mh->header_len = get_gint32 (header_len, buffer->byte_order);
+
+	g_free (header_id);
 	return mh;
 }
 
 
 static int 
-write_mhod_type_3 (Itdb_Thumb *thumb, iPodBuffer *buffer)
+write_mhod_type_1 (gchar *string, iPodBuffer *buffer)
 {
-	MhodHeaderArtworkType3 *mhod;
+	MhodHeaderArtworkType1 *mhod;
 	unsigned int total_bytes;
 	int len;
-	gunichar2 *utf16;
-	int i;
+	int padding;
 
-	g_assert (thumb->filename != NULL);
+	g_assert (string != NULL);
 
-	mhod = (MhodHeaderArtworkType3 *)init_header (buffer, "mhod", 
-						      sizeof (MhodHeaderArtworkType3));
+	mhod = (MhodHeaderArtworkType1 *)init_header (buffer, "mhod", 
+						      sizeof (MhodHeaderArtworkType1));
 	if (mhod == NULL) {
 		return -1;
 	}
-	total_bytes = sizeof (MhodHeaderArtworkType3);
-	mhod->total_len = GINT_TO_LE (total_bytes);
+	total_bytes = sizeof (MhodHeaderArtworkType1);
+	mhod->total_len = get_gint32 (total_bytes, buffer->byte_order);
 	/* Modify header length, since iTunes only puts the length of 
 	 * MhodHeader in header_len
 	 */
-	mhod->header_len = GINT_TO_LE (sizeof (MhodHeader));
-	mhod->type = GINT_TO_LE (3);
-	mhod->mhod_version = GINT_TO_LE (2);
+	mhod->header_len = get_gint32 (sizeof (ArtworkDB_MhodHeader), buffer->byte_order);
+	mhod->unknown3 = get_gint32 (0x01, buffer->byte_order);
+	len = strlen (string);
+	mhod->string_len = get_gint32 (len, buffer->byte_order);
 
-	len = strlen (thumb->filename);
-
-	/* number of bytes of the string encoded in UTF-16 */
-	mhod->string_len = GINT_TO_LE (2*len);
+	padding = 4 - ( (total_bytes + len) % 4 );
+	mhod->padding = padding;
+	mhod->type = get_gint32 (0x0001, buffer->byte_order);
 
 	/* Make sure we have enough free space to write the string */
-	if (ipod_buffer_maybe_grow (buffer, total_bytes + 2*len) != 0) {
+	if (ipod_buffer_maybe_grow (buffer, total_bytes + len + padding ) != 0) {
 		return  -1;
 	}
-	utf16 = g_utf8_to_utf16 (thumb->filename, -1, NULL, NULL, NULL);
-	if (utf16 == NULL) {		
+	memcpy (mhod->string, string, len);
+	total_bytes += len + padding;
+	mhod->total_len = get_gint32 (total_bytes, buffer->byte_order);	
+
+	dump_mhod_type_1 (mhod);
+
+	return total_bytes;
+}
+
+static int 
+write_mhod_type_3 (gchar *string, iPodBuffer *buffer)
+{
+	ArtworkDB_MhodHeaderArtworkType3 *mhod;
+	unsigned int total_bytes;
+	int len;
+	gunichar2 *utf16, *strp;
+	int i, padding;
+
+	g_assert (string != NULL);
+
+	mhod = (ArtworkDB_MhodHeaderArtworkType3 *)
+	    init_header (buffer, "mhod", 
+			 sizeof (ArtworkDB_MhodHeaderArtworkType3));
+	if (mhod == NULL) {
 		return -1;
 	}
-	memcpy (mhod->string, utf16, 2*len);
-	g_free (utf16);
-	for (i = 0; i < len; i++) {
-		mhod->string[i] = GINT_TO_LE (mhod->string[i]);
+	total_bytes = sizeof (ArtworkDB_MhodHeaderArtworkType3);
+	mhod->total_len = get_gint32 (total_bytes, buffer->byte_order);
+	/* Modify header length, since iTunes only puts the length of 
+	 * MhodHeader in header_len
+	 */
+	mhod->header_len = get_gint32 (sizeof (ArtworkDB_MhodHeader), 
+				       buffer->byte_order);
+	mhod->type = get_gint16 (3, buffer->byte_order);
+
+	len = strlen (string);
+
+	/* FIXME: Tidy this up, combine cases more */
+	/* Some magic: endianess-reversed (BE) mobile phones use UTF8
+	 * (version 1) with padding, standard iPods (LE) use UTF16
+	 * (version 2).*/
+	switch (buffer->byte_order)
+	{
+	case G_LITTLE_ENDIAN:
+	    mhod->mhod_version = 2;
+	    /* number of bytes of the string encoded in UTF-16 */
+	    mhod->string_len = get_gint32 (2*len, buffer->byte_order);
+	    padding = 4 - ( (total_bytes + 2*len) % 4 );
+	    mhod->padding = padding; /* high byte is padding length (0-3) */
+
+	    /* Make sure we have enough free space to write the string */
+	    if (ipod_buffer_maybe_grow (buffer, total_bytes + 2*len+padding) != 0) {
+		return  -1;
+	    }
+
+	    utf16 = g_utf8_to_utf16 (string, -1, NULL, NULL, NULL);
+	    if (utf16 == NULL) {		
+		return -1;
+	    }
+	    strp = (gunichar2 *)mhod->string;
+	    memcpy (strp, utf16, 2*len);
+	    g_free (utf16);
+	    for (i = 0; i < len; i++) {
+		strp[i] = get_gint16 (strp[i], buffer->byte_order);
+	    }
+	    total_bytes += 2*len+padding;
+	    break;
+	case G_BIG_ENDIAN:
+	    mhod->mhod_version = 1;
+	    mhod->string_len = get_gint32 (len, buffer->byte_order);
+	    /* pad string if necessary */
+	    /* e.g. len = 7 bytes, len%4 = 3, 4-3=1 -> requires 1 byte
+	       padding */
+	    padding = 4 - ( (total_bytes + len) % 4 );
+	    mhod->padding = padding; /* high byte is padding length (0-3) */
+	    /* Make sure we have enough free space to write the string */
+	    if (ipod_buffer_maybe_grow (buffer, total_bytes + 2*len+padding) != 0) {
+		return  -1;
+	    }
+	    memcpy (mhod->string, string, len);
+	    total_bytes += (len+padding);
 	}
-	total_bytes += 2*len;
-	mhod->total_len = GINT_TO_LE (total_bytes);	
+	mhod->total_len = get_gint32 (total_bytes, buffer->byte_order);
 
 	dump_mhod_type_3 (mhod);
 
@@ -340,7 +426,7 @@ write_mhod_type_3 (Itdb_Thumb *thumb, iPodBuffer *buffer)
 }
 
 static int 
-write_mhni (Itdb_Thumb *thumb, int correlation_id, iPodBuffer *buffer)
+write_mhni (Itdb_DB *db, Itdb_Thumb *thumb, int correlation_id, iPodBuffer *buffer)
 {
 	MhniHeader *mhni;
 	unsigned int total_bytes;
@@ -356,30 +442,41 @@ write_mhni (Itdb_Thumb *thumb, int correlation_id, iPodBuffer *buffer)
 	if (mhni == NULL) {
 		return -1;
 	}
-	total_bytes = GINT_FROM_LE (mhni->header_len);
-	mhni->total_len = GINT_TO_LE (total_bytes);
+	total_bytes =          get_gint32 (mhni->header_len,
+					   buffer->byte_order);
+	mhni->total_len =      get_gint32 (total_bytes,
+					   buffer->byte_order);
+	mhni->correlation_id = get_gint32 (correlation_id,
+					   buffer->byte_order);
+	mhni->image_width =    get_gint16 (thumb->width,
+					   buffer->byte_order);
+	mhni->image_height =   get_gint16 (thumb->height,
+					   buffer->byte_order);
+	mhni->image_size =     get_gint32 (thumb->size,
+					   buffer->byte_order);
+	mhni->ithmb_offset =   get_gint32 (thumb->offset,
+					   buffer->byte_order);
 
-	mhni->correlation_id = GINT_TO_LE (correlation_id);
-	mhni->image_width  = GINT16_TO_LE (thumb->width);
-	mhni->image_height = GINT16_TO_LE (thumb->height);
-	mhni->image_size   = GINT32_TO_LE (thumb->size);
-	mhni->ithmb_offset = GINT32_TO_LE (thumb->offset);
+	mhni->vertical_padding =
+	    get_gint16 (thumb->vertical_padding, buffer->byte_order);
+	mhni->horizontal_padding = 
+	    get_gint16 (thumb->horizontal_padding, buffer->byte_order);
 
 	sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
 	if (sub_buffer == NULL) {
 		return  -1;
 	}
-	bytes_written = write_mhod_type_3 (thumb, sub_buffer);
+	bytes_written = write_mhod_type_3 (thumb->filename, sub_buffer);
 	ipod_buffer_destroy (sub_buffer);
 	if (bytes_written == -1) {
 		return -1;
 	}
 	total_bytes += bytes_written;
-	mhni->total_len = GINT_TO_LE (total_bytes);
+	mhni->total_len = get_gint32 (total_bytes, buffer->byte_order);
 	/* Only update number of children when all went well to try to get
 	 * something somewhat consistent when there are errors 
 	 */
-	mhni->num_children = GINT_TO_LE (1);
+	mhni->num_children = get_gint32 (1, buffer->byte_order);
 	
 	dump_mhni (mhni);
 
@@ -387,9 +484,9 @@ write_mhni (Itdb_Thumb *thumb, int correlation_id, iPodBuffer *buffer)
 }
 
 static int
-write_mhod (Itdb_Thumb *thumb, int correlation_id, iPodBuffer *buffer)
+write_mhod (Itdb_DB *db, Itdb_Thumb *thumb, int correlation_id, iPodBuffer *buffer)
 {
-	MhodHeader *mhod;
+	ArtworkDB_MhodHeader *mhod;
 	unsigned int total_bytes;
 	int bytes_written;
 	iPodBuffer *sub_buffer;
@@ -398,25 +495,26 @@ write_mhod (Itdb_Thumb *thumb, int correlation_id, iPodBuffer *buffer)
 		return -1;
 	}
 
-	mhod = (MhodHeader *)init_header (buffer, "mhod", 
-					  sizeof (MhodHeader));
+	mhod = (ArtworkDB_MhodHeader *)
+	    init_header (buffer, "mhod", 
+			 sizeof (ArtworkDB_MhodHeader));
 	if (mhod == NULL) {
 		return -1;
 	}
-	total_bytes = sizeof (MhodHeader);
-	mhod->total_len = GINT_TO_LE (total_bytes);
-	mhod->type = GINT_TO_LE (MHOD_TYPE_LOCATION);
+	total_bytes = sizeof (ArtworkDB_MhodHeader);
+	mhod->total_len = get_gint32 (total_bytes, buffer->byte_order);
+	mhod->type = get_gint16 (MHOD_TYPE_LOCATION, buffer->byte_order);
 	sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
 	if (sub_buffer == NULL) {
 		return -1;
 	}
-	bytes_written = write_mhni (thumb, correlation_id, sub_buffer);
+	bytes_written = write_mhni (db, thumb, correlation_id, sub_buffer);
 	ipod_buffer_destroy (sub_buffer);
 	if (bytes_written == -1) {
 		return -1;
 	}
 	total_bytes += bytes_written;
-	mhod->total_len = GINT_TO_LE (total_bytes);
+	mhod->total_len = get_gint32 (total_bytes, buffer->byte_order);
 
 	dump_mhod (mhod);
 
@@ -424,44 +522,65 @@ write_mhod (Itdb_Thumb *thumb, int correlation_id, iPodBuffer *buffer)
 }
 
 static int
-write_mhii (Itdb_Track *song, iPodBuffer *buffer)
+write_mhii (Itdb_DB *db, void *data, iPodBuffer *buffer)
 {
 	MhiiHeader *mhii;
 	unsigned int total_bytes;
 	int bytes_written;
 	int num_children;
-	GList *it;
+	GList *it = NULL;
+	Itdb_Track *song;
+	Itdb_Artwork *artwork;
 
 	mhii = (MhiiHeader *)init_header (buffer, "mhii", sizeof (MhiiHeader));
 	if (mhii == NULL) {
 		return -1;
 	}
-	total_bytes = GINT_FROM_LE (mhii->header_len);
-	mhii->song_id = GINT64_TO_LE (song->dbid);
-	mhii->image_id = GUINT_TO_LE (song->artwork->id);
-	mhii->orig_img_size = GINT_TO_LE (song->artwork->artwork_size);
+	total_bytes = get_gint32 (mhii->header_len, buffer->byte_order);
+
+	switch( buffer->db_type) {
+	case DB_TYPE_ITUNES:
+		song = (Itdb_Track *)data;
+		artwork = song->artwork;
+		mhii->song_id = get_gint64 (song->dbid, buffer->byte_order);
+		break;
+	case DB_TYPE_PHOTO:
+		artwork = (Itdb_Artwork *)data;
+		mhii->song_id = get_gint64 (artwork->id + 2, buffer->byte_order);
+		break;
+	default:
+	        g_return_val_if_reached (-1);
+	}
+	mhii->image_id = get_guint32 (artwork->id, buffer->byte_order);
+	mhii->unknown4 = get_gint32 (artwork->unk028, buffer->byte_order);
+	mhii->rating = get_gint32 (artwork->rating, buffer->byte_order);
+	mhii->unknown6 = get_gint32 (artwork->unk036, buffer->byte_order);
+	mhii->orig_date = get_guint32 (artwork->creation_date, buffer->byte_order);
+	mhii->digitized_date = get_guint32 (artwork->digitized_date,
+					    buffer->byte_order);
+	mhii->orig_img_size = get_gint32 (artwork->artwork_size, 
+					  buffer->byte_order);
+	it = artwork->thumbnails;
 	num_children = 0;
-	for (it = song->artwork->thumbnails; it != NULL; it = it->next) {
+	while (it != NULL) {
 		iPodBuffer *sub_buffer;
 		Itdb_Thumb *thumb;
-		const IpodArtworkFormat *img_info;
+		const Itdb_ArtworkFormat *img_info;
 
-		mhii->num_children = GINT_TO_LE (num_children);
-		mhii->total_len = GINT_TO_LE (total_bytes);
+		mhii->num_children = get_gint32 (num_children, 
+						 buffer->byte_order);
+		mhii->total_len = get_gint32 (total_bytes, buffer->byte_order);
 		sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
 		if (sub_buffer == NULL) {
 			return -1;
 		}
 		thumb = (Itdb_Thumb *)it->data;
-		img_info = ipod_get_artwork_info_from_type (
-		    song->itdb->device, thumb->type);
+		img_info = itdb_get_artwork_info_from_type (
+		    db_get_device(db), thumb->type);
 		if (img_info == NULL) {
 			return -1;
 		}
-/* printf("correlation id: %d, type: %d\n", */
-/*        img_info->correlation_id, thumb->type); */
-/* printf("title: %s\n", song->title); */
-		bytes_written = write_mhod (thumb, img_info->correlation_id, 
+		bytes_written = write_mhod (db, thumb, img_info->correlation_id, 
 					    sub_buffer);
 		ipod_buffer_destroy (sub_buffer);
 		if (bytes_written == -1) {
@@ -469,10 +588,11 @@ write_mhii (Itdb_Track *song, iPodBuffer *buffer)
 		}
 		total_bytes += bytes_written;
 		num_children++;
+		it = it->next;
 	}
 
-	mhii->num_children = GINT_TO_LE (num_children);
-	mhii->total_len = GINT_TO_LE (total_bytes);
+	mhii->num_children = get_gint32 (num_children, buffer->byte_order);
+	mhii->total_len = get_gint32 (total_bytes, buffer->byte_order);
 
 	dump_mhii (mhii);
 
@@ -480,9 +600,9 @@ write_mhii (Itdb_Track *song, iPodBuffer *buffer)
 }
 
 static int
-write_mhli (Itdb_iTunesDB *db, iPodBuffer *buffer)
+write_mhli (Itdb_DB *db, iPodBuffer *buffer )
 {
-	GList *it;
+	GList *it = NULL;
 	MhliHeader *mhli;
 	unsigned int total_bytes;
 	int num_thumbs;
@@ -493,57 +613,179 @@ write_mhli (Itdb_iTunesDB *db, iPodBuffer *buffer)
 	}
 
 	num_thumbs = 0;
-	total_bytes = GINT_FROM_LE (mhli->header_len);
-	for (it = db->tracks; it != NULL; it = it->next) {
+	total_bytes = get_gint32 (mhli->header_len, buffer->byte_order);
+	switch (buffer->db_type) {
+	case DB_TYPE_PHOTO:
+		it = db_get_photodb(db)->photos;
+		break;
+	case DB_TYPE_ITUNES:
+		it = db_get_itunesdb(db)->tracks;
+		break;
+	default:
+	        g_return_val_if_reached (-1);
+	}
+	while (it != NULL) {
 		Itdb_Track *song;
 		int bytes_written;
 		iPodBuffer *sub_buffer;
 
-		song = (Itdb_Track*)it->data;
-		if (song->artwork->id == 0) {
-			continue;
+		if (buffer->db_type == DB_TYPE_ITUNES) {
+			song = (Itdb_Track*)it->data;
+			if (!song->artwork || (song->artwork->id == 0)) {
+				it = it->next;
+				continue;
+			}
 		}
 		sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
 		if (sub_buffer == NULL) {
-			continue;
+			break;
 		}
-		bytes_written = write_mhii (song, sub_buffer);
+		bytes_written = write_mhii (db, it->data, sub_buffer);
 		ipod_buffer_destroy (sub_buffer);
 		if (bytes_written != -1) {
 			num_thumbs++;
 			total_bytes += bytes_written;
-		}
+		} 
+		it = it->next;
 	}
-
-	mhli->num_children = GINT_TO_LE (num_thumbs);
-
+	mhli->num_children = get_gint32 (num_thumbs, buffer->byte_order);
 	dump_mhl ((MhlHeader *)mhli, "mhli");
 
 	return total_bytes;
 }
 
 static int
-write_mhla (Itdb_iTunesDB *db, iPodBuffer *buffer)
+write_mhia (gint image_id, iPodBuffer *buffer)
 {
+	MhiaHeader *mhia;
+	unsigned int total_bytes;
+
+
+	mhia = (MhiaHeader *)init_header (buffer, "mhia", 40);
+	if (mhia == NULL) {
+		return -1;
+	}
+
+	mhia->total_len = mhia->header_len;
+	mhia->image_id = get_gint32 (image_id, buffer->byte_order);
+	total_bytes = get_gint32 (mhia->header_len, buffer->byte_order);
+	dump_mhia( mhia );
+	return total_bytes;
+}
+
+static int
+write_mhba (Itdb_PhotoAlbum *album, iPodBuffer *buffer)
+{
+	GList *it;
+	MhbaHeader *mhba;
+	iPodBuffer *sub_buffer;
+	unsigned int total_bytes;
+	unsigned int bytes_written;
+
+	mhba = (MhbaHeader *)init_header (buffer, "mhba", sizeof (MhbaHeader));
+	if (mhba == NULL) {
+		return -1;
+	}
+	mhba->num_mhods = get_gint32(1, buffer->byte_order);
+	mhba->num_mhias = get_gint32(g_list_length (album->members),
+				     buffer->byte_order);
+	mhba->album_id = get_gint32(album->album_id, buffer->byte_order);
+	mhba->unk024 = get_gint32(album->unk024, buffer->byte_order);
+	mhba->unk028 = get_gint16(album->unk028, buffer->byte_order);
+	mhba->album_type = album->album_type;
+	mhba->playmusic = album->playmusic;
+	mhba->repeat = album->repeat;
+	mhba->random = album->random;
+	mhba->show_titles = album->show_titles;
+	mhba->transition_direction = album->transition_direction;
+	mhba->slide_duration = get_gint32(album->slide_duration,
+					  buffer->byte_order);
+	mhba->transition_duration = get_gint32(album->transition_duration,
+					       buffer->byte_order);
+	mhba->unk044 = get_gint32(album->unk044, buffer->byte_order);
+	mhba->unk048 = get_gint32(album->unk048, buffer->byte_order);
+	mhba->song_id = get_gint64(album->song_id, buffer->byte_order);
+	mhba->prev_album_id = get_gint32(album->prev_album_id,
+					 buffer->byte_order);
+
+	total_bytes = get_gint32 (mhba->header_len, buffer->byte_order);
+
+	/* FIXME: Write other mhods */
+	/* Write album title */
+	sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
+	if (sub_buffer == NULL) {
+	    return -1;
+	}
+	bytes_written = write_mhod_type_1 (album->name, sub_buffer);
+	ipod_buffer_destroy (sub_buffer);
+	if (bytes_written == -1) {
+	    return -1;
+	} 
+	total_bytes += bytes_written;
+
+	for (it = album->members; it != NULL; it = it->next) {
+	        Itdb_Artwork *photo = it->data;
+		g_return_val_if_fail (photo, -1);
+
+		sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
+		if (sub_buffer == NULL) {
+		    return -1;
+		}
+		bytes_written = write_mhia (photo->id, sub_buffer);
+		ipod_buffer_destroy (sub_buffer);
+		if (bytes_written == -1) {
+		    return -1;
+		} 
+		total_bytes += bytes_written;
+	}
+	mhba->total_len = get_gint32( total_bytes, buffer->byte_order );
+	dump_mhba ( mhba );
+	return total_bytes;
+}
+
+static int
+write_mhla (Itdb_DB *db, iPodBuffer *buffer)
+{
+	GList *it;
 	MhlaHeader *mhla;
+	iPodBuffer *sub_buffer;
+	unsigned int total_bytes;
 
 	mhla = (MhlaHeader *)init_header (buffer, "mhla", sizeof (MhlaHeader));
 	if (mhla == NULL) {
 		return -1;
 	}
-	
+	total_bytes = get_gint32 (mhla->header_len, buffer->byte_order);
+	if (buffer->db_type == DB_TYPE_PHOTO) {
+	    unsigned int bytes_written;
+
+	    for (it = db_get_photodb(db)->photoalbums; it != NULL; it = it->next) {
+		Itdb_PhotoAlbum *album = (Itdb_PhotoAlbum *)it->data;
+
+		sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
+		if (sub_buffer == NULL) {
+		    return -1;
+		}
+		bytes_written = write_mhba (album, sub_buffer);
+		ipod_buffer_destroy (sub_buffer);
+		if (bytes_written == -1) {
+		    return -1;
+		} 
+		total_bytes += bytes_written;
+		mhla->num_children++;
+	    }
+	}
+
 	dump_mhl ((MhlHeader *)mhla, "mhla");
 
-	return GINT_FROM_LE (mhla->header_len);
+	return total_bytes;
 }
 
-
-
 static int
-write_mhif (Itdb_iTunesDB *db, iPodBuffer *buffer, enum iPodThumbnailType type)
+write_mhif (Itdb_DB *db, iPodBuffer *buffer, enum iPodThumbnailType type)
 {
 	MhifHeader *mhif;
-	const IpodArtworkFormat *img_info;
+	const Itdb_ArtworkFormat *img_info;
 	
 	mhif = (MhifHeader *)init_header (buffer, "mhif", sizeof (MhifHeader));
 	if (mhif == NULL) {
@@ -551,40 +793,52 @@ write_mhif (Itdb_iTunesDB *db, iPodBuffer *buffer, enum iPodThumbnailType type)
 	}
 	mhif->total_len = mhif->header_len;
 	
-	img_info = ipod_get_artwork_info_from_type (db->device, type);
+	img_info = itdb_get_artwork_info_from_type (db_get_device(db), type);
 	if (img_info == NULL) {
 		return -1;
 	}
-
-	mhif->correlation_id = GINT_TO_LE (img_info->correlation_id);
-	mhif->image_size = GINT_TO_LE (img_info->height * img_info->width * 2);
+	mhif->correlation_id = get_gint32 (img_info->correlation_id, 
+					   buffer->byte_order);
+	mhif->image_size = get_gint32 (img_info->height * img_info->width * 2,
+				       buffer->byte_order);
 
 	dump_mhif (mhif);
 	
-	return GINT_FROM_LE (mhif->header_len);
+	return get_gint32 (mhif->header_len, buffer->byte_order);
 }
 
-
 static int
-write_mhlf (Itdb_iTunesDB *db, iPodBuffer *buffer)
+write_mhlf (Itdb_DB *db, iPodBuffer *buffer)
 {
 	MhlfHeader *mhlf;
 	unsigned int total_bytes;
 	int bytes_written;
 	iPodBuffer *sub_buffer;
+	enum iPodThumbnailType thumb_type = IPOD_THUMBNAIL_FULL_SIZE;
 
 	mhlf = (MhlfHeader *)init_header (buffer, "mhlf", sizeof (MhlfHeader));
 	if (mhlf == NULL) {
 		return -1;
 	}
 
-	total_bytes = GINT_FROM_LE (mhlf->header_len);
+	total_bytes = get_gint32 (mhlf->header_len, buffer->byte_order);
 
 	sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
 	if (sub_buffer == NULL) {
 		return -1;
 	}
-	bytes_written = write_mhif (db, sub_buffer, IPOD_THUMBNAIL_FULL_SIZE);
+
+	switch (buffer->db_type) {
+	case DB_TYPE_ITUNES:
+		thumb_type = IPOD_THUMBNAIL_FULL_SIZE;
+		break;
+	case DB_TYPE_PHOTO:
+		thumb_type = IPOD_PHOTO_FULL_SIZE;
+		break;
+	}
+	bytes_written = write_mhif (db
+				  , sub_buffer
+				  , thumb_type);
 	ipod_buffer_destroy (sub_buffer);
 	if (bytes_written == -1) {
 		return -1;
@@ -598,13 +852,24 @@ write_mhlf (Itdb_iTunesDB *db, iPodBuffer *buffer)
 	 * thumbnails: F1016_1.ithmb for the bigger thumbnails (39200 bytes)
 	 * and F1017_1.ithmb for the 'now playing' thumbnails (6272)
 	 */
-	mhlf->num_files = GINT_TO_LE (1);
+	mhlf->num_files = get_gint32 (1, buffer->byte_order);
 
 	sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
 	if (sub_buffer == NULL) {
 		return -1;
 	}
-	bytes_written = write_mhif (db, sub_buffer, IPOD_THUMBNAIL_NOW_PLAYING);
+
+	switch (buffer->db_type) {
+	case DB_TYPE_ITUNES:
+		thumb_type = IPOD_THUMBNAIL_NOW_PLAYING;
+		break;
+	case DB_TYPE_PHOTO:
+		thumb_type = IPOD_PHOTO_LITTLE;
+		break;
+	}
+	bytes_written = write_mhif (db
+				  , sub_buffer
+				  , thumb_type);
 	ipod_buffer_destroy (sub_buffer);
 	if (bytes_written == -1) {
 		return -1;
@@ -618,7 +883,7 @@ write_mhlf (Itdb_iTunesDB *db, iPodBuffer *buffer)
 	 * thumbnails: F1016_1.ithmb for the bigger thumbnails (39200 bytes)
 	 * and F1017_1.ithmb for the 'now playing' thumbnails (6272)
 	 */
-	mhlf->num_files = GINT_TO_LE (2);
+	mhlf->num_files = get_gint32 (2, buffer->byte_order);
 
 	dump_mhl ((MhlHeader *)mhlf, "mhlf");
 
@@ -627,22 +892,22 @@ write_mhlf (Itdb_iTunesDB *db, iPodBuffer *buffer)
 
 
 static int 
-write_mhsd (Itdb_iTunesDB *db, iPodBuffer *buffer, enum MhsdType type)
+write_mhsd (Itdb_DB *db, iPodBuffer *buffer, enum MhsdType type)
 {
-	MhsdHeader *mhsd;
+	ArtworkDB_MhsdHeader *mhsd;
 	unsigned int total_bytes;
 	int bytes_written;
 	iPodBuffer *sub_buffer;
 
 	g_assert (type >= MHSD_TYPE_MHLI);
 	g_assert (type <= MHSD_TYPE_MHLF);
-	mhsd = (MhsdHeader *)init_header (buffer, "mhsd", sizeof (MhsdHeader));
+	mhsd = (ArtworkDB_MhsdHeader *)init_header (buffer, "mhsd", sizeof (ArtworkDB_MhsdHeader));
 	if (mhsd == NULL) {
 		return -1;
 	}
-	total_bytes = GINT_FROM_LE (mhsd->header_len);
-	mhsd->total_len = GINT_TO_LE (total_bytes);
-	mhsd->index = GINT_TO_LE (type);
+	total_bytes = get_gint32 (mhsd->header_len, buffer->byte_order);
+	mhsd->total_len = get_gint32 (total_bytes, buffer->byte_order);
+	mhsd->index = get_gint16 (type, buffer->byte_order);
 	bytes_written = -1;
 
 	sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
@@ -665,7 +930,7 @@ write_mhsd (Itdb_iTunesDB *db, iPodBuffer *buffer, enum MhsdType type)
 		return -1;
 	} else {
 		total_bytes += bytes_written;
-		mhsd->total_len = GINT_TO_LE (total_bytes);
+		mhsd->total_len = get_gint32 (total_bytes, buffer->byte_order);
 	}
 
 	dump_mhsd (mhsd);
@@ -674,7 +939,7 @@ write_mhsd (Itdb_iTunesDB *db, iPodBuffer *buffer, enum MhsdType type)
 }
 
 static int
-write_mhfd (Itdb_iTunesDB *db, iPodBuffer *buffer, int id_max)
+write_mhfd (Itdb_DB *db, iPodBuffer *buffer, int id_max)
 {
 	MhfdHeader *mhfd;
 	unsigned int total_bytes;
@@ -685,11 +950,18 @@ write_mhfd (Itdb_iTunesDB *db, iPodBuffer *buffer, int id_max)
 	if (mhfd == NULL) {
 		return -1;
 	}
-	total_bytes = GINT_FROM_LE (mhfd->header_len);
-	mhfd->total_len = GINT_TO_LE (total_bytes);
-	mhfd->unknown2 = GINT_TO_LE (1);
-	mhfd->unknown4 = GINT_TO_LE (id_max);
-	mhfd->unknown7 = GINT_TO_LE (2);
+	total_bytes = get_gint32 (mhfd->header_len, buffer->byte_order);
+	mhfd->total_len = get_gint32 (total_bytes, buffer->byte_order);
+	switch (buffer->db_type) {
+	case DB_TYPE_PHOTO:
+		mhfd->unknown2 = get_gint32 (2, buffer->byte_order);
+		break;
+	case DB_TYPE_ITUNES:
+		mhfd->unknown2 = get_gint32 (1, buffer->byte_order);
+		break;
+	}
+	mhfd->next_id = get_gint32 (id_max, buffer->byte_order);
+	mhfd->unknown_flag1 = 2;
 	for (i = 1 ; i <= 3; i++) {
 		iPodBuffer *sub_buffer;
 
@@ -703,8 +975,8 @@ write_mhfd (Itdb_iTunesDB *db, iPodBuffer *buffer, int id_max)
 			return -1;
 		}
 		total_bytes += bytes_written;
-		mhfd->total_len = GINT_TO_LE (total_bytes);
-		mhfd->num_children = GINT_TO_LE (i);
+		mhfd->total_len = get_gint32 (total_bytes, buffer->byte_order);
+		mhfd->num_children = get_gint32 (i, buffer->byte_order);
 	}
 
 	dump_mhfd (mhfd);
@@ -733,22 +1005,26 @@ ipod_artwork_db_set_ids (Itdb_iTunesDB *db)
 }
 
 int
-ipod_write_artwork_db (Itdb_iTunesDB *db)
+ipod_write_artwork_db (Itdb_iTunesDB *itdb)
 {
 	iPodBuffer *buf;
 	int bytes_written;
 	int result;
 	char *filename;
 	int id_max;
+	Itdb_DB db;
+
+	db.db_type = DB_TYPE_ITUNES;
+	db.db.itdb = itdb;
 
 	/* First, let's write the .ithmb files, this will create the
 	 * various thumbnails as well */
 
-	itdb_write_ithumb_files (db);
+	itdb_write_ithumb_files (&db);
 	/* Now we can update the ArtworkDB file */
-	id_max = ipod_artwork_db_set_ids (db);
+	id_max = ipod_artwork_db_set_ids (itdb);
 
-	filename = ipod_db_get_artwork_db_path (db->mountpoint);
+	filename = ipod_db_get_artwork_db_path (itdb_get_mountpoint (itdb));
 	if (filename == NULL) {
 		/* FIXME: the iTunesDB will be inconsistent wrt artwork_count
 		 * it might be better to 0 out this field in all tracks
@@ -756,13 +1032,67 @@ ipod_write_artwork_db (Itdb_iTunesDB *db)
 		 */
 		return -1;
 	}
-	buf = ipod_buffer_new (filename);
+	buf = ipod_buffer_new (filename, itdb->device->byte_order, DB_TYPE_ITUNES);
 	if (buf == NULL) {
 		g_print ("Couldn't create %s\n", filename);
 		g_free (filename);
 		return -1;
 	}
-	bytes_written = write_mhfd (db, buf, id_max);
+	bytes_written = write_mhfd (&db, buf, id_max);
+
+	/* Refcount of the mmap buffer should drop to 0 and this should
+	 * sync buffered data to disk
+	 * FIXME: it's probably less error-prone to add a ipod_buffer_mmap_sync
+	 * call...
+	 */
+	ipod_buffer_destroy (buf);
+
+	if (bytes_written == -1) {
+		g_print ("Failed to save %s\n", filename);
+		g_free (filename);
+		/* FIXME: maybe should unlink the file we may have created */
+		return -1;
+	}
+
+	result = truncate (filename, bytes_written);
+	if (result != 0) {
+		g_print ("Failed to truncate %s: %s\n", 
+			 filename, strerror (errno));
+		g_free (filename);	
+		return -1;
+	}
+	g_free (filename);
+	return 0;
+}
+
+int
+ipod_write_photo_db (Itdb_PhotoDB *photodb)
+{
+	iPodBuffer *buf;
+	int bytes_written;
+	int result;
+	char *filename;
+	int id_max;
+	Itdb_DB db;
+
+	db.db_type = DB_TYPE_PHOTO;
+	db.db.photodb = photodb;
+
+	filename = ipod_db_get_photos_db_path (db_get_mountpoint (&db));
+
+	itdb_write_ithumb_files (&db);
+
+	if (filename == NULL) {
+		return -1;
+	}
+	buf = ipod_buffer_new (filename, photodb->device->byte_order, DB_TYPE_PHOTO);
+	if (buf == NULL) {
+		g_print ("Couldn't create %s\n", filename);
+		g_free (filename);
+		return -1;
+	}
+	id_max = itdb_get_free_photo_id( photodb );
+	bytes_written = write_mhfd (&db, buf, id_max);
 
 	/* Refcount of the mmap buffer should drop to 0 and this should
 	 * sync buffered data to disk
@@ -790,7 +1120,13 @@ ipod_write_artwork_db (Itdb_iTunesDB *db)
 }
 #else
 int
-ipod_write_artwork_db (Itdb_iTunesDB *db)
+ipod_write_artwork_db (Itdb_iTunesDB *itdb)
+{
+    return -1;
+}
+
+int
+ipod_write_photo_db (Itdb_PhotoDB *photodb)
 {
     return -1;
 }
